@@ -10,7 +10,7 @@ import ctypes
 import struct
 from weakref import WeakValueDictionary
 from typing import Any
-from collections.abc import Callable
+from collections.abc import Callable, Sequence
 
 import objc
 import Quartz
@@ -33,6 +33,7 @@ from AppKit import (
 
 from xpra.os_util import gi_import
 from xpra.common import roundup
+from xpra.exit_codes import ExitValue
 from xpra.util.env import envint, envbool
 from xpra.util.io import CaptureStdErr
 from xpra.platform.darwin import get_OSXApplication
@@ -320,7 +321,7 @@ def get_display_modes_info(modes) -> dict[int, Any]:
     return {i: get_display_mode_info(mode) for i, mode in enumerate(modes)}
 
 
-def _call_CG_conv(defs: tuple[tuple[str, str, Callable], ...], argument) -> dict[str, Any]:
+def _call_CG_conv(defs: Sequence[tuple[str, str, Callable]], argument) -> dict[str, Any]:
     # utility for calling functions on CG with an argument,
     # then convert the return value using another function
     # missing functions are ignored, and None values are skipped
@@ -488,21 +489,19 @@ OPEN_SIGNAL_WAIT = envint("XPRA_OSX_OPEN_SIGNAL_WAIT", 500)
 
 
 def add_open_handlers(open_file_cb: Callable, open_url_cb: Callable) -> None:
-    def idle_add(fn, *args):
-        GLib.idle_add(fn, *args)
 
     def open_URL(url) -> bool:
         global __osx_open_signal
         __osx_open_signal = True
         log("open_URL(%s)", url)
-        idle_add(open_url_cb, url)
+        GLib.idle_add(open_url_cb, url)
         return True
 
     def open_file(_, filename: str, *args) -> bool:
         global __osx_open_signal
         __osx_open_signal = True
         log("open_file(%s, %s)", filename, args)
-        idle_add(open_file_cb, filename)
+        GLib.idle_add(open_file_cb, filename)
         return True
 
     register_file_handler(open_file)
@@ -702,7 +701,7 @@ class ClientExtras:
         if OSX_FOCUS_WORKAROUND and client:
             def first_ui_received(*_args):
                 enable_focus_workaround()
-                client.timeout_add(OSX_FOCUS_WORKAROUND, disable_focus_workaround)
+                GLib.timeout_add(OSX_FOCUS_WORKAROUND, disable_focus_workaround)
 
             client.connect("first-ui-received", first_ui_received)
         log("ClientExtras.__init__(%s, %s)", client, opts)
@@ -719,13 +718,12 @@ class ClientExtras:
                 log("%s.swap_keys=%s", client.keyboard_helper.keyboard, swap_keys)
                 client.keyboard_helper.keyboard.swap_keys = swap_keys
         if client:
-            self.check_display_timer = client.timeout_add(60 * 1000, self.check_display)
+            self.check_display_timer = GLib.timeout_add(60 * 1000, self.check_display)
 
     def cleanup(self) -> None:
         cdt = self.check_display_timer
-        client = self.client
-        if cdt and client:
-            client.source_remove(cdt)
+        if cdt:
+            GLib.source_remove(cdt)
             self.check_display_timer = 0
         try:
             r = CGDisplayRemoveReconfigurationCallback(self.display_change, self)
@@ -794,7 +792,7 @@ class ClientExtras:
             self.check_display_timer = 0
             return False
 
-    def run(self) -> None:
+    def run(self) -> ExitValue:
         # this is for running standalone
         log("starting console event loop")
         self.event_loop_started = True
@@ -802,6 +800,7 @@ class ClientExtras:
         AppHelper.runConsoleEventLoop(installInterrupt=True)
         # when running from the GTK main loop, we rely on another part of the code
         # to run the event loop for us
+        return 0
 
     def stop(self) -> None:
         if self.event_loop_started:
@@ -810,14 +809,14 @@ class ClientExtras:
             AppHelper.stopEventLoop()
 
 
-def main():
+def main() -> ExitValue:
     from xpra.platform import program_context
     with program_context("OSX Extras"):
         log.enable_debug()
         ce = ClientExtras(None, None)
         ce.check_display()
         ce.ready()
-        ce.run()
+        return ce.run()
 
 
 if __name__ == "__main__":
