@@ -19,7 +19,7 @@ from xpra.client.gtk3.tray_menu import (
     SHOW_QR,
 )
 from xpra.platform.paths import get_icon
-from xpra.platform.darwin.gui import get_OSXApplication
+from xpra.platform.darwin import get_OSXApplication
 from xpra.client.gui import features
 from xpra.log import Logger
 
